import numpy as np

def is_invertible(a):

     return a.shape[0] == a.shape[1] and np.linalg.matrix_rank(a) == a.shape[0]

def C_(A,alpha,p):
  
    
 
    AAT=A@A.conj().T  
     
    Matrix_EYE = np.eye(A.shape[0])
    Temp = (1/alpha) *Matrix_EYE + AAT
    if  is_invertible(Temp):
        INV_Temp = np.linalg.inv(Temp)
    else:
        Temp[Temp == 0]=1e-12
        
        INV_Temp = np.linalg.inv(Temp)   

    C=AAT@INV_Temp    
    
    
    S=C@A
    
    Num=int(np.sum(A==1)/2*p)
    


    
    S_=[]
    
    for i in S:
        
        S_.extend(i)
    

    
    S_sort=np.sort(S_)
    
    Key=S_sort[-Num]
    
    S[S>=Key]=1
    S[S<Key]=0
    
    
    return S



def C(A,alpha):
  
    
 
    AAT=A@A.conj().T  
     
    Matrix_EYE = np.eye(A.shape[0])
    Temp = (1/alpha) *Matrix_EYE + AAT
    if  is_invertible(Temp):
        INV_Temp = np.linalg.inv(Temp)
    else:
        Temp[Temp == 0]=1e-12
        
        INV_Temp = np.linalg.inv(Temp)   

    C=AAT@INV_Temp    
    
    
    S=C@A
    
    return S




def LRTM(A,alpha,beta):
    maxiter=30
    m=len(A)
    n=len(A[0])
    
    
    A=np.array(A)
    W1 = np.random.rand(m,m)
    W2 = np.random.rand(n,n)
    k=1
    
 
    
    
    
    
   
   
    
    AAT=A@A.conj().T  
    ATA=A.conj().T@A
    
    
    
    
    for i in range(maxiter):
        
       
        W1=W1*AAT/((W1@A+A@W2)@A.conj().T+alpha*W1)
        W2=W2*ATA/(A.conj().T@(W1@A+A@W2)+beta*W2)
        
        
       
        W1[W1 == 0]=1e-12
        W2[W2 == 0]=1e-12
       

        
    
    
    S=W1@A+A@W2
    
    return S


from setuptools import setup, find_packages

setup(
    name="openne",
    url="https://github.com/thunlp/OpenNE",
    license="MIT",
    author="THUNLP",
    description="Open Source Network Embedding toolkit",
    packages=find_packages(),
    long_description=open("README.md").read(),
    zip_safe=False,
    setup_requires=[]
)


import pandas as pd
import numpy as np
# from PyBioMed import Pyprotein
# from PyBioMed.PyProtein import CTD
import CTD


pro = pd.read_csv('./data/pro_seq.csv')
seq = pro['seq'].values.astype(str)

protein_descriptor=[]
for i in range(len(seq)):
    ctd = CTD.CalculateCTD(seq[i]).values()
    protein_descriptor.append(list(ctd))

protein_descriptor = np.array(protein_descriptor)
np.savetxt('./data/pro_ctd.txt',protein_descriptor,delimiter=',')


from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_predict
from sklearn.model_selection import KFold
from sklearn.model_selection import cross_val_score
from sklearn.metrics import roc_curve,auc,precision_recall_curve,accuracy_score
import numpy as np
from bayes_opt import BayesianOptimization
from bayes_opt.util import Colours
import random
random.seed(123)
def Classifier_cv(n_estimators, min_samples_split, max_features, data, targets,optimize_type):
    estimator = RandomForestClassifier(
        n_estimators=n_estimators,
        min_samples_split=min_samples_split,
        max_features=max_features,
        n_jobs=-1,
    )
    cval = cross_val_score(estimator, data, targets,
                           scoring=optimize_type, cv=5)
    return cval.mean()



def Best_Param(data, targets,optimize_type):
    def rfc_crossval(n_estimators, min_samples_split, max_features):
        return Classifier_cv(
            n_estimators=int(n_estimators),
            min_samples_split=int(min_samples_split),
            max_features=max(min(max_features, 0.999), 1e-3),
            data=data,
            targets=targets,
            optimize_type=optimize_type,
        )

    optimizer = BayesianOptimization(
        f=rfc_crossval,
        pbounds={
            "n_estimators": (10, 999),
            "min_samples_split": (2, 100),
            "max_features": (0.1, 1.0),
        },
       
        verbose=2
    )
    optimizer.maximize(n_iter=2)
    params=optimizer.max['params']
    modify_para=list(params.values())
    best_param={
                
                'n_estimators':int(modify_para[0]),
                'min_samples_split':int(modify_para[1]),
                'max_features':round(modify_para[2],3),
                'n_jobs':-1,
               }
    print(best_param)
    print(round(optimizer.max['target'],4))
    return best_param









from Embedding.node2vec import *
def node2vec(input_file, output_file, matrix,path_length = 80,num_paths = 10, dim= 64,workers=8, window=10, p=1.0, q=1.0, dw=True):

    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time.time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = Node2vec(graph=g, path_length = path_length,
                                num_paths = num_paths, dim= dim,
                                workers=workers, window=window,p=p, q=q, dw=True)


    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding



# -*- coding: utf-8 -*-
from catboost import CatBoostClassifier
import geatpy as ea 
import numpy as np
import geatpy as ea
from sklearn import svm
from sklearn import preprocessing
from sklearn.model_selection import cross_val_score
from multiprocessing.dummy import Pool as ThreadPool
import random
random.seed(123)
class MyProblem(ea.Problem): 
    def __init__(self,datas,data_targets,optimize_type):
        name = 'MyProblem' 
        
        M = 1
        maxormins = [-1] 
        Dim = 2 
        varTypes = [0, 0] 
        lb = [2**(-8)] * Dim 
        ub = [2**8] * Dim
        lbin = [1] * Dim 
        ubin = [1] * Dim       
        ea.Problem.__init__(self, name, M, maxormins, Dim, varTypes, lb, ub, lbin, ubin)
     
        
        self.data = datas 
        self.dataTarget = data_targets
        self.optimize_type = optimize_type
    
    def aimFunc(self, pop): 
        Vars = pop.Phen 
        pop.ObjV = np.zeros((pop.sizes, 1)) 
        def subAimFunc(i):
            n_estimators = Vars[i, 0]
            learning_rate = Vars[i, 1]
           


            model = CatBoostClassifier(
                                n_estimators=int(n_estimators),
                                learning_rate=learning_rate,

                            )
            model.fit(self.data, self.dataTarget) 
            scores = cross_val_score(model, self.data, self.dataTarget, scoring=self.optimize_type,cv=5) 
            pop.ObjV[i] = scores.mean() 
        pool = ThreadPool(2) 
        pool.map(subAimFunc, list(range(pop.sizes)))
   




def Best_Param(datas,data_targets,optimize_type):


    problem = MyProblem(datas,data_targets,optimize_type) 


    Encoding = 'RI'       
    NIND = 2    

    n_estimators = [10,300] 
    learning_rate = [0,1.0] 
    

    b1 = [1, 1] 
    b2 = [1, 1] 
 
    

  
    ranges=np.vstack([ n_estimators, learning_rate]).T
  
    borders=np.vstack([b1, b2]).T
    varTypes = np.array([0,0]) 



    Field = ea.crtfld(Encoding, varTypes, ranges, borders) 
    population = ea.Population(Encoding, Field, NIND) 
    
    myAlgorithm = ea.soea_DE_rand_1_bin_templet(problem, population) 
    myAlgorithm.MAXGEN = 10 
    myAlgorithm.trappedValue = 1e-6 
    myAlgorithm.maxTrappedCount = 10 
    myAlgorithm.logTras = 1  
    myAlgorithm.verbose = True  
    myAlgorithm.drawing = 1  

    [BestIndi, population] = myAlgorithm.run()  
    BestIndi.save()  
 
    print('Timed：%f s' % myAlgorithm.passTime)
    
    if BestIndi.sizes != 0:
        print('The optimal objective function value is:%s' % BestIndi.ObjV[0][0])
        print('The optimal parameter value is:')
        for i in range(BestIndi.Phen.shape[1]):
            print(BestIndi.Phen[0, i])
    else:
        print('No feasible solution was found.')


    best_param={'n_estimators':int(BestIndi.Phen[0][0]),
                'learning_rate':BestIndi.Phen[0][1],
         
                
               }
    return best_param


#coding=UTF-8
'''
Created on 2016��11��27��

@author: ZWT
'''
import numpy as np
import time

def HPI(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    
    Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train)
    
    deg_row = sum(MatrixAdjacency_Train)
    deg_row.shape = (deg_row.shape[0],1)
    deg_row_T = deg_row.T
    tempdeg = np.minimum(deg_row,deg_row_T)
    
    Matrix_similarity = Matrix_similarity / tempdeg
    
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity



#coding=UTF-8
'''
Created on 2016年11月19日

@author: ZWT
'''
import numpy as np
import time

def Data_Shape(Data):
    List_A = []
    List_B = []
    for row in range(Data.shape[0]):
        List_A.append(Data[row][0])
        List_B.append(Data[row][1])
    List_A = list(set(List_A))
    List_B = list(set(List_B))
    length_A = len(List_A)
    length_B = len(List_B)
 
    MaxNodeNum =  int(max(max(List_A),max(List_B)))+1
    
    return MaxNodeNum
def MatrixAdjacency(MaxNodeNum,Data):
    MatrixAdjacency = np.zeros([MaxNodeNum,MaxNodeNum])
    for col in range(Data.shape[0]):
        i = int(Data[col][0])
        j = int(Data[col][1])
        MatrixAdjacency[i,j] = 1
        MatrixAdjacency[j,i] = 1
    return MatrixAdjacency
def spones(Array):
    for index in range(len(Array)):
        if Array[index] != 0:
            Array[index] = 1
    return Array

def Init(NetFile):
  
    NetData = np.loadtxt(NetFile)
    MaxNodeNum = Data_Shape(NetData)
    MatrixAdjacency_Net = MatrixAdjacency(MaxNodeNum, NetData)
    return MatrixAdjacency_Net,MaxNodeNum
    
def Divide(NetFile,MatrixAdjacency_Net,MaxNodeNum,NetName):
    
    DivideTime_Start = time.clock()
    DivideNum = 0.9
    NetData = np.loadtxt(NetFile)
    permutation = np.random.permutation(NetData.shape[0])
    NetData = NetData[permutation, :]
    TestData = NetData[int(len(NetData)*DivideNum):]
    
    MatrixAdjacency_Train=MatrixAdjacency_Net
    MatrixAdjacency_Test=np.zeros((len(MatrixAdjacency_Net),len(MatrixAdjacency_Net[0])))
    for i in TestData:
            a=int(i[0])
            b=int(i[1])
            MatrixAdjacency_Train[a,b]=0
            MatrixAdjacency_Train[b,a]=0
            MatrixAdjacency_Test[a,b]=1
            MatrixAdjacency_Test[b,a]=1


    
    

   

    DivideTime_End = time.clock()
    
    return MatrixAdjacency_Train,MatrixAdjacency_Test
    
def Init2(Test_File,Train_File):
    print ("DataShape......")
    TrainData = np.loadtxt(Train_File)
    MaxNodeNumTrain = Data_Shape(TrainData)
    TestData = np.loadtxt(Test_File)
    MaxNodeNumTest = Data_Shape(TestData)
    MaxNodeNum = max(MaxNodeNumTrain,MaxNodeNumTest)
    
    MatrixAdjacency_Train = MatrixAdjacency(MaxNodeNum, TrainData)
    MatrixAdjacency_Test = MatrixAdjacency(MaxNodeNum, TestData)
    
    return MatrixAdjacency_Train,MatrixAdjacency_Test,MaxNodeNum





#coding=UTF-8
'''
Created on Nov 29, 2016

@author: ZWT
'''
import numpy as np
import time

def RWR(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    
    Parameter = 0.85
    
    Matrix_TransitionProbobility = MatrixAdjacency_Train / sum(MatrixAdjacency_Train)
    Matrix_EYE = np.eye(MatrixAdjacency_Train.shape[0])
    
    Temp = Matrix_EYE - Parameter * Matrix_TransitionProbobility.T
    INV_Temp = np.linalg.inv(Temp)
    Matrix_RWR = (1 - Parameter) * np.dot(INV_Temp,Matrix_EYE)
    #Matrix_similarity = Matrix_RWR + Matrix_RWR.T
    
    
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity



#coding=UTF-8
'''
Created on 2016年11月15日

@author: ZWT
'''
import numpy as np
import time

def Cn(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train)
    
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity




#coding=UTF-8
'''
Created on 2016��11��27��

@author: ZWT
'''
import numpy as np
import time

def RA(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    
    RA_Train = sum(MatrixAdjacency_Train)
    RA_Train.shape = (RA_Train.shape[0],1)
    MatrixAdjacency_Train_Log = MatrixAdjacency_Train / RA_Train
    MatrixAdjacency_Train_Log = np.nan_to_num(MatrixAdjacency_Train_Log)

    Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train_Log)

    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity


from sklearn.ensemble import AdaBoostClassifier
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_predict
from sklearn.model_selection import KFold
from sklearn.model_selection import cross_val_score
from sklearn.metrics import roc_curve,auc,precision_recall_curve,accuracy_score
import joblib
import numpy as np
from bayes_opt import BayesianOptimization
from bayes_opt.util import Colours
random.seed(123)

def Classifier_cv(n_estimators, learning_rate, data, targets,optimize_type):
    estimator = AdaBoostClassifier(
        n_estimators=n_estimators,
        learning_rate=learning_rate,
      
        n_jobs=-1,
    )
    cval = cross_val_score(estimator, data, targets,
                           scoring=optimize_type, cv=5)
    return cval.mean()



def Best_Param(data, targets,optimize_type):
    def rfc_crossval(n_estimators, learning_rate):
        return Classifier_cv(
            n_estimators=int(n_estimators),
            
            learning_rate=learning_rate,
            data=data,
            targets=targets,
            optimize_type=optimize_type,
        )

    optimizer = BayesianOptimization(
        f=rfc_crossval,
        pbounds={
            "n_estimators": (10, 300),
            "learning_rate":(0,1.0),
        },
       
        verbose=2
    )
    optimizer.maximize(n_iter=30)
    params=optimizer.max['params']
    modify_para=list(params.values())
    best_param={
                'n_estimators':int(modify_para[0]),
                'learning_rate':round(modify_para[1],3),
                'n_jobs':-1,
               }
    print(best_param)
    print(round(optimizer.max['target'],4))
    return best_param








#coding=UTF-8
'''
Created on Nov 30, 2016

@author: ZWT
'''
import numpy as np
import time

# def Cos(MatrixAdjacency_Train):
#     similarity_StartTime = time.clock()
#     
#     Matrix_D = np.diag(sum(MatrixAdjacency_Train))
#     Matrix_D.shape = (Matrix_D.shape[0],1)
#     Matrix_Laplacian = Matrix_D - MatrixAdjacency_Train
#     INV_Matrix_Laplacian  = np.linalg.pinv(Matrix_Laplacian)
#     
#     Array_Diag = np.diag(INV_Matrix_Laplacian)
#     Matrix_ONE = np.ones([MatrixAdjacency_Train.shape[0],MatrixAdjacency_Train.shape[0]])
# 
#     Matrix_similarity = (Matrix_D.T,Matrix_D)
#     print Matrix_similarity
#     Matrix_similarity = np.sqrt(Matrix_similarity)
#     print Matrix_similarity
#     Matrix_similarity = INV_Matrix_Laplacian / Matrix_similarity
#     Matrix_similarity
#     Matrix_similarity = np.nan_to_num(Matrix_similarity)
#     
#     similarity_EndTime = time.clock()
#     print "    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime)
#     return Matrix_similarity

def Cos(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
     
    Matrix_D = np.diag(sum(MatrixAdjacency_Train))
    Matrix_Laplacian = Matrix_D - MatrixAdjacency_Train
    INV_Matrix_Laplacian  = np.linalg.pinv(Matrix_Laplacian)
     
    Array_Diag = np.diag(INV_Matrix_Laplacian)
    Matrix_ONE = np.ones([MatrixAdjacency_Train.shape[0],MatrixAdjacency_Train.shape[0]])
    Matrix_Diag = Array_Diag * Matrix_ONE
 
    Matrix_similarity = INV_Matrix_Laplacian/((Matrix_Diag * Matrix_Diag.T) ** 0.5)

    Matrix_similarity = np.nan_to_num(Matrix_similarity)
     
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity


from Embedding.hope import *

def Hope(input_file, output_file,matrix, dim=64):
    

    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time.time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = HOPE(graph=g, d=dim)


    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding

# if __name__ == '__main__':
#     input_file = '../embeddingdata/input.txt'
#     matrix=np.loadtxt(input_file, dtype=float)
#     Embeddingdata = Hope(matrix, dim=64)
#     # print(Embeddingdata)


import setuptools

with open("README.md", "r") as fh:
  long_description = fh.read()

setuptools.setup(
  name="autobio-tool",
  version="0.1.9",
  author="kaizheng&&haoyuanli",
  author_email="a1242884508@gmail.com",
  description="A tool for prediction of molecules associations",
  url="https://github.com/haohaodaren/autobio-tool.git",
  packages=setuptools.find_packages(),
  classifiers=[
  "Programming Language :: Python :: 3",
  "License :: OSI Approved :: MIT License",
  "Operating System :: OS Independent",
  ],
)



#coding=UTF-8
'''
Created on Nov 29, 2016

@author: ZWT
'''
import numpy as np
import time

def ACT(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    
    Matrix_D = np.diag(sum(MatrixAdjacency_Train))
    Matrix_Laplacian = Matrix_D - MatrixAdjacency_Train
    INV_Matrix_Laplacian  = np.linalg.pinv(Matrix_Laplacian)
    
    Array_Diag = np.diag(INV_Matrix_Laplacian)
    Matrix_ONE = np.ones([MatrixAdjacency_Train.shape[0],MatrixAdjacency_Train.shape[0]])
    Matrix_Diag = Array_Diag * Matrix_ONE

    Matrix_similarity = Matrix_Diag + Matrix_Diag.T - (2 * Matrix_Laplacian)
   
    Matrix_similarity = Matrix_ONE / Matrix_similarity
    Matrix_similarity = np.nan_to_num(Matrix_similarity)
    
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity


from Embedding.grarep import *

def Grarep(input_file, output_file, matrix,Kstep= 4, dim=64):


    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time.time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = GraRep(graph=g, Kstep= Kstep, dim=dim)

    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding




import tensorflow as tf
import numpy as np


def uniform(shape, scale=0.05, name=None):
    """Uniform init."""
    initial = tf.random_uniform(
        shape, minval=-scale, maxval=scale, dtype=tf.float32)
    return tf.Variable(initial, name=name)


def glorot(shape, name=None):
    """Glorot & Bengio (AISTATS 2010) init."""
    init_range = np.sqrt(6.0/(shape[0]+shape[1]))
    initial = tf.random_uniform(
        shape, minval=-init_range, maxval=init_range, dtype=tf.float32)
    return tf.Variable(initial, name=name)


def zeros(shape, name=None):
    """All zeros."""
    initial = tf.zeros(shape, dtype=tf.float32)
    return tf.Variable(initial, name=name)


def ones(shape, name=None):
    """All ones."""
    initial = tf.ones(shape, dtype=tf.float32)
    return tf.Variable(initial, name=name)




import numpy as np
import random
from graph import *
import node2vec
import time
import ast

class Graph(object):
    def __init__(self):
        self.G = None
        self.look_up_dict = {}
        self.look_back_list = []
        self.node_size = 0
        
    def encode_node(self):
        look_up = self.look_up_dict
        look_back = self.look_back_list
        for node in self.G.nodes():
            look_up[node] = self.node_size
            look_back.append(node)
            self.node_size += 1
            self.G.nodes[node]['status'] = ''
    def read_edgelist(self, filename, weighted=False, directed=False):
        self.G = nx.DiGraph()

        if directed:
            def read_unweighted(l):
                src, dst = l.split()
                self.G.add_edge(src, dst)
                self.G[src][dst]['weight'] = 1.0

            def read_weighted(l):
                src, dst, w = l.split()
                self.G.add_edge(src, dst)
                self.G[src][dst]['weight'] = float(w)
        else:
            def read_unweighted(l):
                src, dst = l.split()
                self.G.add_edge(src, dst)
                self.G.add_edge(dst, src)
                self.G[src][dst]['weight'] = 1.0
                self.G[dst][src]['weight'] = 1.0

            def read_weighted(l):
                src, dst, w = l.split()
                self.G.add_edge(src, dst)
                self.G.add_edge(dst, src)
                self.G[src][dst]['weight'] = float(w)
                self.G[dst][src]['weight'] = float(w)
        fin = open(filename, 'r')
        func = read_unweighted
        if weighted:
            func = read_weighted
        while 1:
            l = fin.readline()
            if l == '':
                break
            func(l)
        fin.close()
        self.encode_node()


def deepwalkembedding(input_file, output_file, path_length, num_paths, dim, workers, window):
    t1 = time.time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = node2vec.Node2vec(graph=g, path_length = path_length,
                                num_paths = num_paths, dim= dim,
                                workers=workers, window=window, dw=True)

    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)

if __name__ == "__main__":
    
    random.seed(32)
    np.random.seed(32)
    input_file = '../../data/cora/cora_edgelist.txt'
    output_file = '../../data/cora/cora_embedding.txt'
    path_length = 80
    num_paths = 10
    dim = 64
    workers = 8
    window = 10
    dw=True
    deepwalkembedding(input_file, output_file, path_length, num_paths, dim, workers, window)



import  csv
from numpy import *
import numpy as np
import csv
import random
import pandas as pd
from sklearn.model_selection import StratifiedKFold
from sklearn.metrics import roc_curve,auc,precision_recall_curve
def ReadCsv(fileName):
    SaveList=[]
    csv_reader = csv.reader(open(fileName))
    for row in csv_reader:
        SaveList.append(list(map(float,row)))
    return SaveList
def SaveCsv(data, fileName):
    with open(fileName, "w", newline='') as csvfile:
        writer = csv.writer(csvfile)
        writer.writerows(data)
    return

def loadmodel(modelname):

    if modelname=='RandomForestClassifier':

        model = joblib.load('./model/RandomForestClassifier.auto')

    return model

def shuffle(data, targets):
    data = np.array(data)
    targets = np.array(targets)
    permutation = np.random.permutation(targets.shape[0])
    data = data[permutation, :]
    targets = targets[permutation]
    return data, targets


import numpy as np
import pickle as pkl
import networkx as nx
import scipy.sparse as sp
from scipy.sparse.linalg.eigen.arpack import eigsh
import sys


def parse_index_file(filename):
    """Parse index file."""
    index = []
    for line in open(filename):
        index.append(int(line.strip()))
    return index


def sample_mask(idx, l):
    """Create mask."""
    mask = np.zeros(l)
    mask[idx] = 1
    return np.array(mask, dtype=np.bool)


def load_data(dataset_str):
    """Load data."""
    names = ['x', 'y', 'tx', 'ty', 'allx', 'ally', 'graph']
    objects = []
    for i in range(len(names)):
        with open("data/ind.{}.{}".format(dataset_str, names[i]), 'rb') as f:
            if sys.version_info > (3, 0):
                objects.append(pkl.load(f, encoding='latin1'))
            else:
                objects.append(pkl.load(f))

    x, y, tx, ty, allx, ally, graph = tuple(objects)
    test_idx_reorder = parse_index_file(
        "data/ind.{}.test.index".format(dataset_str))
    test_idx_range = np.sort(test_idx_reorder)

    if dataset_str == 'citeseer':
        # Fix citeseer dataset (there are some isolated nodes in the graph)
        # Find isolated nodes, add them as zero-vecs into the right position
        test_idx_range_full = range(
            min(test_idx_reorder), max(test_idx_reorder)+1)
        tx_extended = sp.lil_matrix((len(test_idx_range_full), x.shape[1]))
        tx_extended[test_idx_range-min(test_idx_range), :] = tx
        tx = tx_extended
        ty_extended = np.zeros((len(test_idx_range_full), y.shape[1]))
        ty_extended[test_idx_range-min(test_idx_range), :] = ty
        ty = ty_extended

    features = sp.vstack((allx, tx)).tolil()
    features[test_idx_reorder, :] = features[test_idx_range, :]
    adj = nx.adjacency_matrix(nx.from_dict_of_lists(graph))

    labels = np.vstack((ally, ty))
    labels[test_idx_reorder, :] = labels[test_idx_range, :]

    idx_test = test_idx_range.tolist()
    idx_train = range(len(y))
    idx_val = range(len(y), len(y)+500)

    train_mask = sample_mask(idx_train, labels.shape[0])
    val_mask = sample_mask(idx_val, labels.shape[0])
    test_mask = sample_mask(idx_test, labels.shape[0])

    y_train = np.zeros(labels.shape)
    y_val = np.zeros(labels.shape)
    y_test = np.zeros(labels.shape)
    y_train[train_mask, :] = labels[train_mask, :]
    y_val[val_mask, :] = labels[val_mask, :]
    y_test[test_mask, :] = labels[test_mask, :]

    return adj, features, y_train, y_val, y_test, train_mask, val_mask, test_mask


def sparse_to_tuple(sparse_mx):
    """Convert sparse matrix to tuple representation."""
    def to_tuple(mx):
        if not sp.isspmatrix_coo(mx):
            mx = mx.tocoo()
        coords = np.vstack((mx.row, mx.col)).transpose()
        values = mx.data
        shape = mx.shape
        return coords, values, shape

    if isinstance(sparse_mx, list):
        for i in range(len(sparse_mx)):
            sparse_mx[i] = to_tuple(sparse_mx[i])
    else:
        sparse_mx = to_tuple(sparse_mx)

    return sparse_mx


def preprocess_features(features):
    """Row-normalize feature matrix and convert to tuple representation"""
    rowsum = np.array(features.sum(1))
    r_inv = np.power(rowsum, -1).flatten()
    r_inv[np.isinf(r_inv)] = 0.
    r_mat_inv = sp.diags(r_inv)
    features = sp.coo_matrix(features)
    features = r_mat_inv.dot(features)
    return sparse_to_tuple(features)


def normalize_adj(adj):
    """Symmetrically normalize adjacency matrix."""
    adj = sp.coo_matrix(adj)
    rowsum = np.array(adj.sum(1))
    d_inv_sqrt = np.power(rowsum, -0.5).flatten()
    d_inv_sqrt[np.isinf(d_inv_sqrt)] = 0.
    d_mat_inv_sqrt = sp.diags(d_inv_sqrt)
    return adj.dot(d_mat_inv_sqrt).transpose().dot(d_mat_inv_sqrt).tocoo()


def preprocess_adj(adj):
    """Preprocessing of adjacency matrix for simple GCN model and conversion to tuple representation."""
    adj_normalized = normalize_adj(adj + sp.eye(adj.shape[0]))
    return sparse_to_tuple(adj_normalized)


def construct_feed_dict(features, support, labels, labels_mask, placeholders):
    """Construct feed dictionary."""
    feed_dict = dict()
    feed_dict.update({placeholders['labels']: labels})
    feed_dict.update({placeholders['labels_mask']: labels_mask})
    feed_dict.update({placeholders['features']: features})
    feed_dict.update({placeholders['support'][i]: support[i]
                      for i in range(len(support))})
    feed_dict.update({placeholders['num_features_nonzero']: features[1].shape})
    return feed_dict


def chebyshev_polynomials(adj, k):
    """Calculate Chebyshev polynomials up to order k. Return a list of sparse matrices (tuple representation)."""
    print("Calculating Chebyshev polynomials up to order {}...".format(k))

    adj_normalized = normalize_adj(adj)
    laplacian = sp.eye(adj.shape[0]) - adj_normalized
    largest_eigval, _ = eigsh(laplacian, 1, which='LM')
    scaled_laplacian = (
        2. / largest_eigval[0]) * laplacian - sp.eye(adj.shape[0])

    t_k = list()
    t_k.append(sp.eye(adj.shape[0]))
    t_k.append(scaled_laplacian)

    def chebyshev_recurrence(t_k_minus_one, t_k_minus_two, scaled_lap):
        s_lap = sp.csr_matrix(scaled_lap, copy=True)
        return 2 * s_lap.dot(t_k_minus_one) - t_k_minus_two

    for i in range(2, k+1):
        t_k.append(chebyshev_recurrence(t_k[-1], t_k[-2], scaled_laplacian))

    return sparse_to_tuple(t_k)



from Embedding.node2vec import *

def Deepwalk(input_file, output_file, matrix,path_length = 80,num_paths = 10, dim= 64,workers=8, window=10):
    
    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time.time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = Node2vec(graph=g, path_length = path_length,
                                num_paths = num_paths, dim= dim,
                                workers=workers, window=window, dw=True)


    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding

# if __name__ == '__main__':
#     input_file = '../embeddingdata/input.txt'
#     matrix=np.loadtxt(input_file, dtype=float)
#     Embeddingdata = Deepwalk(matrix,path_length = 80,num_paths = 10, dim= 64,workers=8, window=10)
#     # print(Embeddingdata)



from __future__ import print_function
import random
import numpy as np
import multiprocessing


def deepwalk_walk_wrapper(class_instance, walk_length, start_node):
    class_instance.deepwalk_walk(walk_length, start_node)


class BasicWalker:
    def __init__(self, G, workers):
        self.G = G.G
        self.node_size = G.node_size
        self.look_up_dict = G.look_up_dict

    def deepwalk_walk(self, walk_length, start_node):
        '''
        Simulate a random walk starting from start node.
        '''
        G = self.G
        look_up_dict = self.look_up_dict
        node_size = self.node_size

        walk = [start_node]

        while len(walk) < walk_length:
            cur = walk[-1]
            cur_nbrs = list(G.neighbors(cur))
            if len(cur_nbrs) > 0:
                walk.append(random.choice(cur_nbrs))
            else:
                break
        return walk

    def simulate_walks(self, num_walks, walk_length):
        '''
        Repeatedly simulate random walks from each node.
        '''
        G = self.G
        walks = []
        nodes = list(G.nodes())
        print('Walk iteration:')
        for walk_iter in range(num_walks):
            # pool = multiprocessing.Pool(processes = 4)
            print(str(walk_iter+1), '/', str(num_walks))
            random.shuffle(nodes)
            for node in nodes:
                # walks.append(pool.apply_async(deepwalk_walk_wrapper, (self, walk_length, node, )))
                walks.append(self.deepwalk_walk(
                    walk_length=walk_length, start_node=node))
            # pool.close()
            # pool.join()
        # print(len(walks))
        return walks


class Walker:
    def __init__(self, G, p, q, workers):
        self.G = G.G
        self.p = p
        self.q = q
        self.node_size = G.node_size
        self.look_up_dict = G.look_up_dict

    def node2vec_walk(self, walk_length, start_node):
        '''
        Simulate a random walk starting from start node.
        '''
        G = self.G
        alias_nodes = self.alias_nodes
        alias_edges = self.alias_edges
        look_up_dict = self.look_up_dict
        node_size = self.node_size

        walk = [start_node]

        while len(walk) < walk_length:
            cur = walk[-1]
            cur_nbrs = list(G.neighbors(cur))
            if len(cur_nbrs) > 0:
                if len(walk) == 1:
                    walk.append(
                        cur_nbrs[alias_draw(alias_nodes[cur][0], alias_nodes[cur][1])])
                else:
                    prev = walk[-2]
                    pos = (prev, cur)
                    next = cur_nbrs[alias_draw(alias_edges[pos][0],
                                               alias_edges[pos][1])]
                    walk.append(next)
            else:
                break

        return walk

    def simulate_walks(self, num_walks, walk_length):
        '''
        Repeatedly simulate random walks from each node.
        '''
        G = self.G
        walks = []
        nodes = list(G.nodes())
        print('Walk iteration:')
        for walk_iter in range(num_walks):
            print(str(walk_iter+1), '/', str(num_walks))
            random.shuffle(nodes)
            for node in nodes:
                walks.append(self.node2vec_walk(
                    walk_length=walk_length, start_node=node))

        return walks

    def get_alias_edge(self, src, dst):
        '''
        Get the alias edge setup lists for a given edge.
        '''
        G = self.G
        p = self.p
        q = self.q

        unnormalized_probs = []
        for dst_nbr in G.neighbors(dst):
            if dst_nbr == src:
                unnormalized_probs.append(G[dst][dst_nbr]['weight']/p)
            elif G.has_edge(dst_nbr, src):
                unnormalized_probs.append(G[dst][dst_nbr]['weight'])
            else:
                unnormalized_probs.append(G[dst][dst_nbr]['weight']/q)
        norm_const = sum(unnormalized_probs)
        normalized_probs = [
            float(u_prob)/norm_const for u_prob in unnormalized_probs]

        return alias_setup(normalized_probs)

    def preprocess_transition_probs(self):
        '''
        Preprocessing of transition probabilities for guiding the random walks.
        '''
        G = self.G

        alias_nodes = {}
        for node in G.nodes():
            unnormalized_probs = [G[node][nbr]['weight']
                                  for nbr in G.neighbors(node)]
            norm_const = sum(unnormalized_probs)
            normalized_probs = [
                float(u_prob)/norm_const for u_prob in unnormalized_probs]
            alias_nodes[node] = alias_setup(normalized_probs)

        alias_edges = {}
        triads = {}

        look_up_dict = self.look_up_dict
        node_size = self.node_size
        for edge in G.edges():
            alias_edges[edge] = self.get_alias_edge(edge[0], edge[1])

        self.alias_nodes = alias_nodes
        self.alias_edges = alias_edges

        return


def alias_setup(probs):
    '''
    Compute utility lists for non-uniform sampling from discrete distributions.
    Refer to https://hips.seas.harvard.edu/blog/2013/03/03/the-alias-method-efficient-sampling-with-many-discrete-outcomes/
    for details
    '''
    K = len(probs)
    q = np.zeros(K, dtype=np.float32)
    J = np.zeros(K, dtype=np.int32)

    smaller = []
    larger = []
    for kk, prob in enumerate(probs):
        q[kk] = K*prob
        if q[kk] < 1.0:
            smaller.append(kk)
        else:
            larger.append(kk)

    while len(smaller) > 0 and len(larger) > 0:
        small = smaller.pop()
        large = larger.pop()

        J[small] = large
        q[large] = q[large] + q[small] - 1.0
        if q[large] < 1.0:
            smaller.append(large)
        else:
            larger.append(large)

    return J, q


def alias_draw(J, q):
    '''
    Draw sample from a non-uniform discrete distribution using alias sampling.
    '''
    K = len(J)

    kk = int(np.floor(np.random.rand()*K))
    if np.random.rand() < q[kk]:
        return kk
    else:
        return J[kk]



import tensorflow as tf


def masked_softmax_cross_entropy(preds, labels, mask):
    """Softmax cross-entropy loss with masking."""
    loss = tf.nn.softmax_cross_entropy_with_logits(logits=preds, labels=labels)
    mask = tf.cast(mask, dtype=tf.float32)
    mask /= tf.reduce_mean(mask)
    loss *= mask
    return tf.reduce_mean(loss)


def masked_accuracy(preds, labels, mask):
    """Accuracy with masking."""
    correct_prediction = tf.equal(tf.argmax(preds, 1), tf.argmax(labels, 1))
    accuracy_all = tf.cast(correct_prediction, tf.float32)
    mask = tf.cast(mask, dtype=tf.float32)
    mask /= tf.reduce_mean(mask)
    accuracy_all *= mask
    return tf.reduce_mean(accuracy_all)



import csv
csv.field_size_limit(500 * 1024 * 1024)

import os
import time
import numpy as np
# import pandas as pd
import math
import random
from MyKmer import MyKmer


def RNA_kmer(BiomoleculesSequence):
    BiomoleculesKmer = []
    counter = 0
    while counter < len(BiomoleculesSequence):
        try:       
            sequence = MyKmer(BiomoleculesSequence[counter][1])
            pair = []
            pair.append(BiomoleculesSequence[counter][0])
            pair.extend(sequence)
            BiomoleculesKmer.append(pair)
        except:   
            pair = []
            pair.append(BiomoleculesSequence[counter][0])
            pair.extend(np.zeros((1, 64), dtype=int)[0])           
            BiomoleculesKmer.append(pair)
        print(counter)
        counter = counter + 1

    return BiomoleculesKmer